----------------------------------------------------------------------------
-- +-----------+----------------+
-- | Mode19A.e | Version 1.202B |    For Mode 19 Only
-- +-----------+----------------+    For Use with Euphoria 2.0 +
-- |     2-D Graphics Engine    |
-- |   Created and Produced by  |     
-- |    Hollow Horse Software   |
-- |          (C) 1997          |
-- +----------------------------+     
----------------------------------------------------------------------------

include machine.e
include image.e

global integer SCREENS
atom wait_retrace
atom dis_code, sdis_code
sequence code
atom copy_address,single_ad
global sequence screen
global sequence lookup

lookup = repeat(0, 201)

SCREENS = 0
for y = 1 to 201 do       --make a lookup table, 
    lookup[y] = SCREENS   --may make a difference in large images
    SCREENS = SCREENS + 320
end for

SCREENS = 0
screen = {#A0000}

----------------------------------------------------------------------------
global function Detect_Max_Virtual_Pages()
sequence sFree_mem
atom data_struct, cFree_mem
atom free_block, max_unlocked, page_size, 
     total_free, total_physical
         
    data_struct = allocate(#30)
    if not data_struct then
        return -1
    end if
    mem_set(data_struct,0,#30)

    sFree_mem = {
    #60,                              --   pushad
    #06,                              --   push es
    #1E,                              --   push ds
    #07,                              --   pop  es
    #BF}&int_to_bytes(data_struct)&{  --   mov edi, data_struct
    #B8,0,5,0,0,                      --   mov eax, #500
    #CD,#31,                          --   int #31
    #07,                              --   pop es
    #61,                              --   popad
    #C3}                               --   ret

    cFree_mem = allocate(length(sFree_mem))
    if not cFree_mem then
        free(data_struct)
        return -1
    end if
    poke(cFree_mem,sFree_mem)
    call(cFree_mem)

    free_block = bytes_to_int(peek({data_struct,4}))
    max_unlocked = bytes_to_int(peek({data_struct+4,4}))
    page_size = free_block/max_unlocked
    total_free = bytes_to_int(peek({data_struct+#14,4}))
    total_physical = bytes_to_int(peek({data_struct+#18,4}))
    free(cFree_mem)
    free(data_struct)
    return floor(((total_physical*page_size)/2)/64000)
end function        
----------------------------------------------------------------------------
global procedure vpixel(sequence p, object s, integer u)
    --add 1 to u because 0 is real screen
    --plot the pixel or pixels
    p = floor(p)
    poke(lookup[p[2] + 1] + p[1] + screen[u + 1], s)
end procedure--vpixel
----------------------------------------------------------------------------
global function vget_pixel(sequence p, integer u)
    --add 1 to u because 0 is real screen
    --return the pixel or pixels
    p[1..2] = floor(p[1..2])
    if length(p) > 2 then
        return peek({lookup[p[2] + 1] + p[1] + screen[u + 1], p[3]})
    else
        return peek(lookup[p[2] + 1] + p[1] + screen[u + 1])
    end if
end function--vget_pixel
----------------------------------------------------------------------------
global function Bmp2Mem(sequence bmp)
    atom pos
    integer temp, width, height
    sequence header
--  header = {mem_add, width, height}

    header = {}
    temp = 0
    if atom(bmp[1][1]) then
        bmp = {bmp}
        temp = 1
    end if
    for a = 1 to length(bmp) do
        width = length(bmp[a][1])
        height = length(bmp[a])
        pos = allocate(width * height)
        if pos = 0 then
            return {-1}
        end if    
        header = append(header, {pos, width, height})

        for z = 1 to height do
            --z = where at in height
            --pos = where (height * width)
            --pos = pos + address
            --poke bmp at height z into memory at address pos
            poke(pos, bmp[a][z])
            pos = pos + width
        end for
    end for

    if temp then
        return header[1]
    else
        return header
    end if
end function--Bmp2Mem
----------------------------------------------------------------------------
global function Mem2Bmp(sequence header)
    atom pos
    integer width, height
    sequence bmp
--  header = {mem_add, width, height}

    pos = header[1]
    width = header[2]
    height = header[3]
    bmp = repeat(0, height)
    
    for z = 1 to height do
        --z = where at in height
        --pos = where (height * width)
        --pos = pos + address
        --peek bmp at height z from memory at address pos
        bmp[z] = peek({pos, width})
        pos = pos + width
    end for
    return bmp
end function--Mem2Bmp
----------------------------------------------------------------------------
global function set_screens(integer x)
    atom max
    -- if x is less than 0 then value invalid
    if x < 0 then
        puts(2, "Invalid value for set_screens()")
        abort(1)
    end if
    max = Detect_Max_Virtual_Pages()
    if max and x >=max then
        x = max - 1
    end if
    if SCREENS < x then  --add virtual screens
        screen = screen & repeat(0, x - SCREENS)
        for i = SCREENS + 2 to x + 1 do
            screen[i] = allocate(64000)
            SCREENS = SCREENS + 1
            if screen[i] = 0 then
                screen = screen[1..SCREENS]
                SCREENS = SCREENS - 1
                return SCREENS
            end if
        end for
    elsif SCREENS > x then -- remove virtual screens
        --adding 2 to x instead of one because
        --1 is real screen
        --adding 1 to SCREENS for same reason
        for i = x + 2 to SCREENS + 1 do
            free(screen[i])
        end for
        screen = screen[1..x + 1]
        SCREENS = x
    end if
    return SCREENS
end function--set_screens
----------------------------------------------------------------------------

global procedure screen_copy(integer dest, integer source)
    --add 1 to dest and source because
    --0 = real screen
    mem_copy(screen[dest + 1], screen[source + 1], 64000)
end procedure--screen_copy
----------------------------------------------------------------------------

global procedure CLS(integer u, integer c)
    --add 1 to u because 0 is real screen
    mem_set(screen[u + 1], c, 64000)
end procedure--CLS
----------------------------------------------------------------------------

global procedure fdisplay(sequence p, sequence s, integer u)
    atom temp, pos
    integer width
        pos = lookup[p[2] + 1] + p[1] + screen[u + 1]
        temp = s[1]
        width = s[2] 
        if width = 320 then
            mem_copy(pos, temp, lookup[s[3] + 1])
        else
            for z = 1 to s[3] do
                mem_copy(pos, temp, width)
                pos = pos + 320
                temp = temp + width
            end for
        end if
end procedure--fdisplay
----------------------------------------------------------------------------
global procedure display(sequence p, sequence s, integer u)
    atom scr, bmp
    integer x1, y1, bh, bw

    p = floor(p)
    if atom(s[1]) then
        scr = screen[u + 1]
        bmp = s[1]
        x1 = p[1]  y1 = p[2]
        bw = s[2]
        bh = s[3] 
        poke4(dis_code + 65, {scr, y1, x1, bh, bw, bmp})
        call(dis_code)
    end if
end procedure--display
----------------------------------------------------------------------------
global procedure cdisplay(sequence xy, sequence image,integer c,integer u)
 atom scr, bmp
 integer x1, y1, bh, bw
 if c < 0 then c = 0 end if
 if c > 255 then c = 255 end if
 scr = screen[u + 1]
 bmp = image[1]
 x1 = xy[1]  y1 = xy[2]
 bw = image[2]
 bh = image[3] 
 
 poke4(sdis_code + 5, scr)
 poke4(sdis_code + 10, y1)
 poke4(sdis_code + 25, x1)
 poke4(sdis_code + 32, bmp)
 poke4(sdis_code + 37, bh)
 poke4(sdis_code + 42, bw)
 poke(sdis_code + 54, c)
 call(sdis_code)
end procedure--mdisplay
-----------------------------------------------------------------------------
procedure set_emc_address(atom address)
    poke4(copy_address+2,address)
end procedure
-----------------------------------------------------------------------------
procedure se_mem_copy(integer kind,atom src,atom dest,atom width,
                      atom height,atom src_add,atom dest_add)
    set_emc_address(single_ad)
    poke(single_ad,kind)
    poke4(single_ad+1, src)
    poke4(single_ad+5, dest)
    poke4(single_ad+9, width)
    poke4(single_ad+13,height)
    poke4(single_ad+17,src_add)
    poke4(single_ad+21,dest_add)
    call(copy_address)
end procedure
-----------------------------------------------------------------------------
global procedure display_clip(sequence p, sequence s, integer u)
    atom e_p,s_a,d_a,work, src
    integer width, height, xpos, ypos, ytemp, xtemp
    work = screen[u+1]
    src = s[1]
    width = s[2]
    height = s[3]
    p = floor(p)
    xpos = p[1]
    ypos = p[2]
    xtemp = xpos + width
    ytemp = ypos + height
    if ypos < 0 then
      elsif xpos < 0 then
      elsif ytemp > 200 then
      elsif xtemp > 320 then
      else
         poke4(dis_code + 65, {work, ypos, xpos, height, width, src})
         call(dis_code)
         return
    end if
    
    s_a = 0
    d_a = 320 - width
    if ypos < 0 then         -- off the top?
        height = height + ypos -- -work
        if height < 1 then
            return
        end if
        src = src - width * ypos --work
        ypos = 0
    end if
    if ytemp > 200 then      -- off the bottom?
        if ypos > 199 then
            return
        end if
        height = 200 - ypos
    end if
    if xpos < 0 then         -- off the left?
        s_a = -xpos
        if s_a >= width then
            return
        end if
        width = width + xpos 
        src = src - xpos 
        d_a = d_a - xpos
        xpos = 0
    end if
    
    if xtemp > 320 then   --off the right
        if xpos > 319 then    
            return
        end if    
        work = xtemp - 320
        width = 320 - xpos
        s_a = s_a + work
        d_a = d_a + work
    end if
        
    e_p = screen[u+1] + ypos * 320 + xpos
    se_mem_copy(2,src,e_p,width,height,s_a,d_a)
end procedure--display_clip()
----------------------------------------------------------------------------
global procedure cdisplay_clip(sequence p, sequence s, integer c, integer u)
    atom pos
    integer top, bottom, left, right
    integer x, y, width, height
    p = floor(p)
    top = 0
    bottom = 0
    left = 0
    right = 0
    x = p[1]
    y = p[2]
    width = s[2]
    height = s[3]
        if y < 0 then
        elsif x < 0 then
        elsif (y + height) > 200 then
        elsif (x + width) > 320 then
        else
            cdisplay(p, s, c, u)
            return
        end if
        if y > 199 then
            return
        elsif x > 319 then
            return
        end if
        if y < 0 then
            if (-y) < height then
                top = y
            else
                return
            end if
            y = 0
        end if
        if x < 0 then
            if (-x) < width then
                left = x
            else
                return
            end if
            x = 0
        end if
        bottom = y + height - 200
        if bottom < 0 then
            bottom = 0
        end if
        right = x + width - 320
        if right < 0 then
            right = 0
        end if
        if (width + left - right) > 0 then
            if left or right then
                pos = s[1] - (top * width) - left
                for a = 1 - top to height - bottom do
                    cdisplay({x, y}, {pos, width + left - right, 1},c, u)
                    pos = pos + width
                    y = y + 1
                end for
            else
                pos = s[1] - (top * width)
                cdisplay({x, y}, {pos, width, height + top - bottom},c, u)
            end if
        end if
end procedure--cdisplay_clip()
-----------------------------------------------------------------------------
global procedure fdisplay_clip(sequence p, sequence s, integer u)
    atom e_p,s_a,d_a,work, src, ypos, width
    integer height, xpos, xtemp, ytemp
    src = s[1]
    width = s[2]
    height = s[3]
    p = floor(p)
    xpos = p[1]
    ypos = p[2]
    ytemp = ypos + height
    xtemp = xpos + width
    if ypos < 0 then
      elsif xpos < 0 then
      elsif ytemp > 200 then
      elsif xtemp > 320 then
      else
         fdisplay(p, s, u)
      return
    end if
    s_a = 0
    d_a = 320 - width
    if ypos < 0 then         -- off the top?
        height = height + ypos -- -work
        if height < 1 then
            return
        end if
        src = src - width * ypos --work
        ypos = 0
    end if
    if ytemp > 200 then      -- off the bottom?
        if ypos > 199 then
            return
        end if
        height = 200 - ypos
    end if
    if xpos < 0 then         -- off the left?
        s_a = -xpos
        if s_a >= width then
            return
        end if
        width = width + xpos 
        src = src - xpos 
        d_a = d_a - xpos
        xpos = 0
    end if
    
    if xtemp > 320 then   --off the right
        if xpos > 319 then    
            return
        end if    
        work = xtemp - 320
        width = 320 - xpos
        s_a = s_a + work
        d_a = d_a + work
    end if
    
    e_p = screen[u+1] + ypos * 320 + xpos
    se_mem_copy(1,src,e_p,width,height,s_a,d_a)
end procedure--fdisplay_clip()
-----------------------------------------------------------------------------
global procedure SetTypeRate(integer rate)
    sequence reg_list
    if rate < 0 then
        rate = 0
    elsif rate > 31 then
        rate = 31
    end if
    reg_list = repeat(0,10)
    reg_list[REG_AX] = #305
    reg_list[REG_BX] = and_bits(rate,3) * 256 + and_bits(rate,31)
    reg_list = dos_interrupt(#16, reg_list)
end procedure--SetTypeRate
----------------------------------------------------------------------------
procedure Create_Retrace_Code()
sequence wait_retrace_code

wait_retrace_code = {
           #50,          -- PUSH EAX
           #52,          -- PUSH EDX
           #BA,#DA,3,0,0,-- MOV EDX, 0x03DA
           #EC,          -- IN AL, DX
           #24,#08,      -- AND AL, 0x08
           #75,#FB,      -- JNZ $-5
           #EC,          -- IN AL, DX
           #24,#08,      -- AND AL, 0x08
           #74,#FB,      -- JZ $-5
           #5A,          -- POP EDX
           #58,          -- POP EAX
           #C3 }         -- RET

    wait_retrace = allocate(length(wait_retrace_code))
    poke(wait_retrace, wait_retrace_code)
    lock_memory(wait_retrace, length(wait_retrace_code))
end procedure--Create_Retrace_Code
----------------------------------------------------------------------------
global procedure Wait_Retrace()
    call(wait_retrace)
end procedure--Wait_Retrace
----------------------------------------------------------------------------
function abs(integer x)
    if x < 0 then
        x = -x
    end if
    return x
end function--abs
---------------------------------------------------------------------------
global procedure vline(sequence xy, sequence xy2, integer c, integer u)
integer deltax, deltay, numpixels,
        d, dinc1, dinc2,
        x, xinc1, xinc2,
        y, yinc1, yinc2
    
    if u = 0 then
        draw_line(c, {xy, xy2})
        return
    end if
    
    deltax = abs(xy2[1] - xy[1])
    deltay = abs(xy2[2] - xy[2])

    if deltay = 0 then
        vpixel(xy, repeat(c, deltax), u)
        return
    end if

    xinc2 = 1
    yinc2 = 1

    if deltax >= deltay then
        numpixels = deltax + 1
        d = (2 * deltay) - deltax
        dinc1 = deltay * 2
        dinc2 = (deltay - deltax) * 2
        xinc1 = 1
        yinc1 = 0
    else
        numpixels = deltay + 1
        d = (2 * deltax) - deltay
        dinc1 = deltax * 2
        dinc2 = (deltax - deltay) * 2
        xinc1 = 0
        yinc1 = 1
    end if
    if xy[1] > xy2[1] then
        xinc1 = -xinc1
        xinc2 = -xinc2
    end if
    if xy[2] > xy2[2] then
        yinc1 = -yinc1
        yinc2 = -yinc2
    end if
    x = xy[1]
    y = xy[2]
    
    for i = 1 to numpixels do
        vpixel({x, y}, c, u)
        if d < 0 then
            d = d + dinc1
            x = x + xinc1
            y = y + yinc1
        else
            d = d + dinc2
            x = x + xinc2
            y = y + yinc2
        end if
    end for                                 
end procedure--vline
----------------------------------------------------------------------------
procedure Create_Display_Code()
sequence display_code

display_code = {
 #60,                                    --pusha
 #06,                                    --push es
 #1E,                                    --push ds
 #07,                                    --pop es
 #BE, #00, #00, #00, #00,                --mov esi,0x0
 #AD,                                    --lodsd
 #89, #C7,                               --mov edi,eax
 #AD,                                    --lodsd
 #89, #C1,                               --mov ecx,eax
 #C1, #E0, #06,                          --shl eax,0x6
 #C1, #E1, #08,                          --shl ecx,0x8
 #01, #C1,                               --add ecx,eax
 #AD,                                    --lodsd
 #01, #C1,                               --add ecx,eax
 #01, #CF,                               --add edi,ecx
 #AD,                                    --lodsd
 #89, #C3,                               --mov ebx,eax
 #AD,                                    --lodsd
 #89, #C1,                               --mov ecx,eax
 #AD,                                    --lodsd
 #89, #C6,                               --mov esi,eax
 #89, #CA,                               --mov edx,ecx
 #AC,                                    --lodsb
 #3C, #00,                               --cmp al,0x0
 #74, #02,                               --je 0x2e
 #AA,                                    --stosb
 #4F,                                    --dec edi
 #47,                                    --inc edi
 #E0, #F6,                               --loopne 0x27
 #81, #C7, #40, #01, #00, #00,           --add edi,0x140
 #29, #D7,                               --sub edi,edx
 #89, #D1,                               --mov ecx,edx
 #4B,                                    --dec ebx
 #75, #E9,                               --jne 0x27
 #07,                                    --pop es
 #61,                                    --popa
 #C3                                     --ret
}
& repeat(0, 24)

dis_code = allocate(length(display_code))
poke(dis_code, display_code)
poke4(dis_code + 5, dis_code + 65)
lock_memory(dis_code, length(display_code))
end procedure--Create_Display_Code
-----------------------------------------------------------------------------
procedure Create_SDisplay_Code()
sequence sdisplay_code

sdisplay_code = {
 #60,                                    --pusha
 #06,                                    --push es
 #1E,                                    --push ds
 #07,                                    --pop es
 #BF, #00, #00, #00, #00,                --mov edi,0x0
 #B8, #00, #00, #00, #00,                --mov eax,0x0
 #89, #C1,                               --mov ecx,eax
 #C1, #E0, #06,                          --shl eax,0x6
 #C1, #E1, #08,                          --shl ecx,0x8
 #01, #C8,                               --add eax,ecx
 #05, #00, #00, #00, #00,                --add eax,0x0
 #01, #C7,                               --add edi,eax
 #BE, #00, #00, #00, #00,                --mov esi,0x0
 #BB, #00, #00, #00, #00,                --mov ebx,0x0
 #B9, #00, #00, #00, #00,                --mov ecx,0x0
 #89, #CA,                               --mov edx,ecx
 #AC,                                    --lodsb
 #3C, #00,                               --cmp al,0x0
 #74, #04,                               --je 0x39
 #B0, #00,                               --mov al,0x0
 #AA,                                    --stosb
 #4F,                                    --dec edi
 #47,                                    --inc edi
 #E0, #F4,                               --loopne 0x30
 #81, #C7, #40, #01, #00, #00,           --add edi,0x140
 #29, #D7,                               --sub edi,edx
 #89, #D1,                               --mov ecx,edx
 #4B,                                    --dec ebx
 #75, #E7,                               --jne 0x30
 #07,                                    --pop es
 #61,                                    --popa
 #C3}                                     --ret


sdis_code = allocate(length(sdisplay_code))
mem_set(sdis_code, 0, length(sdisplay_code))
poke(sdis_code, sdisplay_code)
lock_memory(sdis_code, length(sdisplay_code))
end procedure--Create_SDisplay_Code
----------------------------------------------------------------------------
----------------------------------------------------------------------------
function set_up_emc() -- Micheal Bolin
    copy_address=allocate(length(code))
    single_ad=allocate(26)
    if copy_address and single_ad then
        poke(copy_address,code)
        poke(single_ad,repeat(0,26))
        return 1
    else
        return 0
    end if
end function
-----------------------------------------------------------------------------
global procedure copy_region(integer kind, sequence xy, sequence xy2, 
                             integer src, integer dest)
    integer width, height, offset, start
    atom scrto, scrfrom
    width = xy2[1] - xy[1]
    height = xy2[2] - xy[2]
    offset = 320 - width
    start = (320 * xy[2]) + xy[1]
    scrto = screen[dest+1] + start 
    scrfrom = screen[src+1] + start
    se_mem_copy(kind, scrfrom, scrto, width, height, offset, offset)
end procedure    
----------------------------------------------------------------------------

code={
        #60,#B8,#00,#00,#00,#00,#8A,#18,#80,#FB,#00,#74,#45,#50,#8B,#50,#05,
        #8B,#48,#09,#89,#0D,#2A,#02,#00,#00,#8B,#48,#0D,#89,#0D,#2E,#02,#00,
        #00,#8B,#48,#11,#89,#0D,#32,#02,#00,#00,#8B,#48,#15,#89,#0D,#36,#02,
        #00,#00,#8B,#40,#01,#80,#FB,#01,#74,#17,#80,#FB,#02,#74,#22,#E8,#47,
        #01,#00,#00,#58,#05,#19,#00,#00,#00,#E9,#B4,#FF,#FF,#FF,#61,#C3,#E8,
        #1B,#00,#00,#00,#58,#05,#19,#00,#00,#00,#E9,#A2,#FF,#FF,#FF,#E8,#CE,
        #00,#00,#00,#58,#05,#19,#00,#00,#00,#E9,#92,#FF,#FF,#FF,#8B,#2D,#2E,
        #02,#00,#00,#39,#D0,#7C,#4C,#8B,#0D,#2A,#02,#00,#00,#81,#F9,#04,#00,
        #00,#00,#7C,#23,#81,#E9,#04,#00,#00,#00,#8B,#18,#05,#04,#00,#00,#00,
        #89,#1A,#81,#C2,#04,#00,#00,#00,#81,#E9,#04,#00,#00,#00,#79,#E9,#81,
        #C1,#04,#00,#00,#00,#74,#09,#8A,#18,#40,#88,#1A,#42,#49,#75,#F7,#03,
        #05,#32,#02,#00,#00,#03,#15,#36,#02,#00,#00,#4D,#75,#B5,#C3,#E8,#16,
        #01,#00,#00,#2D,#03,#00,#00,#00,#81,#EA,#03,#00,#00,#00,#8B,#0D,#2A,
        #02,#00,#00,#81,#F9,#04,#00,#00,#00,#7C,#23,#81,#E9,#04,#00,#00,#00,
        #8B,#18,#2D,#04,#00,#00,#00,#89,#1A,#81,#EA,#04,#00,#00,#00,#81,#E9,
        #04,#00,#00,#00,#79,#E9,#81,#C1,#04,#00,#00,#00,#05,#03,#00,#00,#00,
        #81,#C2,#03,#00,#00,#00,#81,#F9,#00,#00,#00,#00,#74,#09,#8A,#18,#48,
        #88,#1A,#4A,#49,#75,#F7,#2B,#05,#32,#02,#00,#00,#2B,#15,#36,#02,#00,
        #00,#4D,#75,#99,#C3,#8B,#2D,#2E,#02,#00,#00,#39,#D0,#7C,#24,#8B,#0D,
        #2A,#02,#00,#00,#8A,#18,#40,#80,#FB,#00,#74,#02,#88,#1A,#42,#49,#75,
        #F2,#03,#05,#32,#02,#00,#00,#03,#15,#36,#02,#00,#00,#4D,#75,#DD,#C3,
        #E8,#7B,#00,#00,#00,#8B,#0D,#2A,#02,#00,#00,#8A,#18,#48,#80,#FB,#00,
        #74,#02,#88,#1A,#4A,#49,#75,#F2,#2B,#05,#32,#02,#00,#00,#2B,#15,#36,
        #02,#00,#00,#4D,#75,#DD,#C3,#8B,#2D,#2E,#02,#00,#00,#39,#D0,#7C,#24,
        #8B,#0D,#2A,#02,#00,#00,#80,#3A,#00,#75,#04,#8A,#18,#88,#1A,#42,#40,
        #49,#75,#F2,#03,#05,#32,#02,#00,#00,#03,#15,#36,#02,#00,#00,#4D,#75,
        #DD,#C3,#E8,#24,#00,#00,#00,#8B,#0D,#2A,#02,#00,#00,#80,#3A,#00,#75,
        #04,#8A,#18,#88,#1A,#4A,#48,#49,#75,#F2,#2B,#05,#32,#02,#00,#00,#2B,
        #15,#36,#02,#00,#00,#4D,#75,#DD,#C3,#89,#C6,#A1,#2A,#02,#00,#00,#03,
        #05,#32,#02,#00,#00,#8B,#3D,#2E,#02,#00,#00,#4F,#52,#F7,#E7,#5A,#03,
        #05,#2A,#02,#00,#00,#48,#01,#F0,#50,#A1,#2A,#02,#00,#00,#03,#05,#36,
        #02,#00,#00,#8B,#3D,#2E,#02,#00,#00,#4F,#52,#F7,#E7,#5A,#03,#05,#2A,
        #02,#00,#00,#48,#01,#D0,#89,#C2,#58,#C3,#00,#00,#00,#00,#00,#00,#00,
        #00,#00,#00,#00,#00,#00,#00,#00,#00}
----------------------------------------------------------------------------     
if set_up_emc() then
    Create_Retrace_Code()
    Create_Display_Code()
    Create_SDisplay_Code()
else
    puts(2, "Not enough free memory. Program Aborted.")
    abort(1)
end if    
----------------------------------------------------------------------------
--end code

-- In Euphoria 2.1 this file (unless modified) is FREE - 0 statements.
with 373801342 -- delete this statement if it causes an error

